#define  _main
#include "defines.h"
#include "LPC17xx.H"
#include <RTL.h>
#include <stdio.h>
#include "LED.h"
#include "main.h"
#include "GLCD.h" 
/**************************************************************************/
/* Task1 is used to send a mail message to Task2. 												*/
/* The message contains two variables which are used to switch the 				*/
/*upper and lower nibbles of the LED bank. Once all the LED's are					*/
/* on an event flag is set on Task 3 which switches off the LED's 				*/
/* The process then restarts																							*/
/**************************************************************************/

__task void Task1 (void){
	
OS_RESULT Error;
MailMessage* msgPtr;	
uint32_t ledLower,ledUpper,i;
	
	while(1){
		ledLower=0;ledUpper = 7;																							//Set the inital values for the LED bank
		for(i=0;i<4;i++){																											
			os_dly_wait(10);
			Error = os_mbx_check(MsgBoxTask2);																	//check if there is a free mail slot
			if (Error!=0) {																											// If there is a free mailslot
				msgPtr = 	_alloc_box(mpoolTask2);																	//Allocate a mailslot for this message
				msgPtr->ledLowerNibble = ledLower;																//Put the LED data into the message
				msgPtr->ledUpperNibble = ledUpper;
				os_mbx_send(MsgBoxTask2, msgPtr, 0xFFFF); 									   		//Post the message
			}else {																															   
				os_evt_set(MAILBOX_OVERFLOW,t_Error);															//If there is no free mailslot signal an error
			}
			ledLower = ledLower+1;																							//Update the led data for the next message
			ledUpper = ledUpper-1;
			}
		os_dly_wait(10);
		os_evt_set(0x0001,t_Task3);																						//Signal task3 to clear the LED's
}}

/******************************************************************************/
/* Task2 receives a mail message, checks that the LED data is within range    */
/* and then writes the data to the bank of LED's. If the data is out of range */
/* the error task is signaled with an error code                              */
/******************************************************************************/

__task void Task2 (void){
MailMessage* message;
	while(1) {
		os_mbx_wait (MsgBoxTask2, (void **)&message, 0xFFFF);									//wait for a message to arrive
		if(message->ledLowerNibble<=3 ){																			//check the data is within range
			LED_On(message->ledLowerNibble);																		//Write the data to the LED bank
		}else{
			os_evt_set(LOWER_LED_PARAMETER_INVALID,t_Error);}										//If data is out of range signal an error
		
		if(message->ledUpperNibble<=7 && message->ledUpperNibble>=4){					//Check the upper nibble data is within range
		LED_On(message->ledUpperNibble);																			//Write the data to the LED bank
		}else{
			os_evt_set(UPPER_LED_PARAMETER_INVALID,t_Error);}										//If data is out of range signal an error
		_free_box(mpoolTask2, message);																				// Free the mailslot
}}

/****************************************************************************************/
/* Task3 waits for an event flag to be set. When this happens it switches off all the   */
/* LEDs                                                                                 */
/****************************************************************************************/

__task void Task3 (void){
while(1)
{
	os_evt_wait_or(0x0001,0xFFFF);																				//Wait here untl the event flag is set
	LED_Out(0x00);																												//Switch off all the LEDs
}
}

/*************************************************************************************/
/* The init task creates the mail queues and the tasks                               */
/* It also signals the error tas to print a sign on message and then deletes itself  */
/*************************************************************************************/

__task void init (void) {

	
	LED_Init ();
	
	_init_box(mpoolTask2, sizeof(mpoolTask2), sizeof(MailMessage)); 	//Create the mailqueue
	os_mbx_init(MsgBoxTask2, sizeof(MsgBoxTask2));
	
	t_Task1 	= os_tsk_create(Task1, 10); 														//Start the tasks
	t_Task2 	= os_tsk_create(Task2, 10); 
	t_Task3 	= os_tsk_create(Task3, 10); 
	t_Error		=	os_tsk_create(errorHandler, 1);
	t_Test		= os_tsk_create(testTask, 1);
	os_evt_set(RTX_STARTED,t_Error);																	//Signal the error task
	os_tsk_delete_self ();																						//End this task	
}


int main (void) {
	GLCD_Init();                               /* Initialize graphical LCD      */

  GLCD_Clear(White);                         /* Clear graphical LCD display   */
  GLCD_SetBackColor(Blue);
  GLCD_SetTextColor(White);
  GLCD_DisplayString(0, 0, __FI, "Testing RTX projects");
  GLCD_DisplayString(1, 0, __FI, "   using the ITM    ");		
	
	os_sys_init(init);																								// start RTX and launch the init task
}
